use Test::More;
use Readonly;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::File qw( GetFileContent );
use DE_EPAGES::AvaTax::API::XML::IsAuthorizedResponseHandler;
use DE_EPAGES::Core::API::PerlTools qw (IsA);
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );

use strict;

Readonly::Scalar my $TEST_TRANSACTION_ID => 17508882;
sub testParse {
        my $Response = DE_EPAGES::AvaTax::API::XML::IsAuthorizedResponseHandler->importXML(
            ${GetFileContent('IsAuthorizedResponse.xml')}
        );

    ok(IsA($Response, 'DE_EPAGES::AvaTax::API::DTO::IsAuthorizedResult'), 'Response is a IsAuthorizedResult');
    cmp_ok($Response->TransactionId,'==',$TEST_TRANSACTION_ID,'TransactionID: '.$Response->TransactionId);
    cmp_ok($Response->ResultCode,'eq','Success','ResultCode: '.$Response->ResultCode);
    cmp_ok(scalar @{ $Response->Messages } , '==', 1, '1 Message');
    my $Message = $Response->popMessage;
    cmp_ok($Message->Summary, 'eq','Message 1 Summary','Summary');
    cmp_ok($Message->Details, 'eq','Message 1 Details','Details');
    cmp_ok($Message->HelpLink, 'eq','http://www.Message1Helplink.com/','Summary');
    cmp_ok($Message->RefersTo, 'eq','','RefersTo');
    cmp_ok($Message->Severity, 'eq','Debug','Severity');
    cmp_ok($Message->Source, 'eq','?','Source');

}

RunScript(
    'Sub' => sub {
        plan skip_all => 'No such file: IsAuthorizedResponse.xml' unless (-r 'IsAuthorizedResponse.xml');
        eval {
            testParse();
        };
        if (ExistsError()) {
            my $Error = GetError();
            fail('ERROR: '. substr($Error->shortMessage, 0, 4096) . "\n");
        }
        done_testing();
    }
);
