#========================================================================================
# package      DE_EPAGES::AvaTax::t::unit::AdjustTaxSender
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  test DE_EPAGES::AvaTax::API::AdjustTaxSender
#========================================================================================
package DE_EPAGES::AvaTax::t::unit::AdjustTaxSender;
use base DE_EPAGES::Core::API::Test;
use Test::More;
use DE_EPAGES::Test::Mock::Mockify qw ( GetParametersFromMockifyCall );
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::MockObject;
use DE_EPAGES::Core::API::TestParameter qw (TestObject);
use DE_EPAGES::Test::API::TestTools qw (TestExistsMethod);
use DE_EPAGES::Core::API::PerlTools qw ( UsePackage );
use DE_EPAGES::AvaTax::API::Constants qw ( AVATAX_DOCUMENTTYPE_SalesInvoice AVATAX_ADJUSTTAX_SUCCESS);
use strict;

sub test {
    my $self = shift;
    $self->test_sendGetTaxRequestWhenCommitted();
    done_testing();
}

sub test_sendGetTaxRequestWhenCommitted {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $AdjustTaxRequestBuilder = $self->_createAdjustTaxRequestBuilder();
    my $AdjustTaxSender = $self->_createAdjustTaxSender($AdjustTaxRequestBuilder);

    my $LineItemContainer = $self->_createLineItemContainer();
    $AdjustTaxSender->sendAdjustTax($self->_createOrderExtractor($LineItemContainer));

    my $aParameters = GetParametersFromMockifyCall($AdjustTaxRequestBuilder,'buildRequest');
    my $DTO_GetTaxRequest = $aParameters->[0];
    is($DTO_GetTaxRequest->DocCode(), 'Test-Doc-Code-123' , "$SubTestName - tests if DTO is set correctly (DocCode)");
    is($DTO_GetTaxRequest->DocType(), AVATAX_DOCUMENTTYPE_SalesInvoice , "$SubTestName - tests if DTO is set correctly (DocType)");
    is($DTO_GetTaxRequest->Commit(), 'true' , "$SubTestName - tests if DTO is set correctly (Commit)");
    is($DTO_GetTaxRequest->CustomerUsageType(), 'somekindofexemption' , "$SubTestName - tests if DTO is set correctly (ExemptionNo)");

    is($LineItemContainer->get('AvaTax_StatusResponse'), AVATAX_ADJUSTTAX_SUCCESS, "$SubTestName - tests if Status is set correctly");
    is($LineItemContainer->get('AvaTax_StatusMessage'), 'message', "$SubTestName - tests if StatusMesssage is set correctly");
    is($LineItemContainer->get('AvaTax_TimeStamp'), 'now', "$SubTestName - tests if TimeStamp is set correctly");
}

#========================================================================================
# Helpers
#========================================================================================
sub _createAdjustTaxSender {
    my $self = shift;
    my ($AdjustTaxRequestBuilder) = @_;

    my $MockifyObject = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::AvaTax::API::AdjustTaxSender',
        []
    );

    my $aParams = [{'object' => 'DE_EPAGES::Shop::API::Object::Shop'}];
    $MockifyObject->addMockWithReturnValueAndParameterCheck('_connectToTaxService', $self->_createRemoteConnector(), $aParams);
    $MockifyObject->addMockWithReturnValue('_createGetTaxRequestBuilder', $self->_createGetTaxRequestBuilder());
    $MockifyObject->addMockWithReturnValue('_createAdjustTaxRequestBuilder', $AdjustTaxRequestBuilder);
    $MockifyObject->addMockWithReturnValue('_getCurrentDateTime', 'now');
    $MockifyObject->addMockWithReturnValueAndParameterCheck('_createResponseParser', $self->_createResponseParser(), [
        {'object' => 'DE_EPAGES::AvaTax::API::DTO::AdjustTaxResult'}
    ]);

    return $MockifyObject->getMockObject();
}

sub _createShop {
    my $self = shift;

    my $MockedObject = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {});
    return $MockedObject;
}

sub _createExemptionAttribute {
    my $self = shift;

    my $hAtributes = {'Alias' => 'somekindofexemption'};
    return  DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::PreDefAttribute',$hAtributes);
}

sub _createOrderExtractor {
    my $self = shift;
    my ($LineItemContainer) = @_;

    my $MockifyObject = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::AvaTax::API::OrderExtractor',
        []
    );
    $MockifyObject->addMockWithReturnValue('getShop', $self->_createShop());
    $MockifyObject->addMockWithReturnValue('getLineItemContainer', $LineItemContainer);
    $MockifyObject->addMockWithReturnValue('getAvataxTaxExemption', $self->_createExemptionAttribute());

    return $MockifyObject->getMockObject();
}

sub _createLineItemContainer {
    my $self = shift;
    my $hAttributes = {
        'AvaTax_DocCode' => 'Test-Doc-Code-123',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes);
}

sub _createRemoteConnector {
    my $self = shift;

    UsePackage( 'DE_EPAGES::AvaTax::API::RemoteTaxSvc' );
    TestExistsMethod('DE_EPAGES::AvaTax::API::RemoteTaxSvc', 'adjustTax');
    my $Mock = Test::MockObject->new();
    $Mock->mock('adjustTax', sub {
        my $MockedSelf = shift;
        my ($Request) = @_;
        TestObject('adjustTaxRequest object', $Request, 'DE_EPAGES::AvaTax::API::DTO::AdjustTaxRequest');
        return $self->_createResponse();
    });

    return $Mock;
}

sub _createGetTaxRequestBuilder {
    my $self = shift;

    my $MockedObject = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::AvaTax::API::GetTaxRequestBuilder',
        []
    );
    my $GetTaxRequest = DE_EPAGES::AvaTax::API::DTO::GetTaxRequest->new();
    $MockedObject->addMockWithReturnValueAndParameterCheck(
        'buildRequest',
        $GetTaxRequest,
        [{'object' => 'DE_EPAGES::AvaTax::API::OrderExtractor'}]
    );
    return $MockedObject->getMockObject();
}

sub _createAdjustTaxRequestBuilder {
    my $self = shift;

    my $MockifyObject = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::AvaTax::API::AdjustTaxRequestBuilder',
        []
    );
    my $AdjustTaxRequest = DE_EPAGES::AvaTax::API::DTO::AdjustTaxRequest->new();

    my $aParams = [{'object' => 'DE_EPAGES::AvaTax::API::DTO::GetTaxRequest'}];
    $MockifyObject->addMockWithReturnValueAndParameterCheck('buildRequest',$AdjustTaxRequest,$aParams);
    return $MockifyObject->getMockObject();
}

sub _createResponse {
    my $self = shift;
    return DE_EPAGES::AvaTax::API::DTO::AdjustTaxResult->new();
}

sub _createResponseParser {
    my $self = shift;
    my $MockifyObject = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::AvaTax::API::ResponseParser',
        []
    );
    $MockifyObject->addMockWithReturnValue('getStatusAdjustTax', AVATAX_ADJUSTTAX_SUCCESS);
    $MockifyObject->addMockWithReturnValue('getStatusMessage', 'message');
    return $MockifyObject->getMockObject();
}

__PACKAGE__->runScript();