#========================================================================================
# package      DE_EPAGES::AvaTax::t::unit::HashCodeBuilder
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for HashCodeBuilder class
#========================================================================================
package DE_EPAGES::AvaTax::t::unit::HashCodeBuilder;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::AvaTax::API::DTO::BaseAddress;
use DE_EPAGES::AvaTax::API::DTO::Line;
use DE_EPAGES::AvaTax::API::OrderExtractor;
use DE_EPAGES::Test::Mock::Mockify;

use DE_EPAGES::AvaTax::API::HashCodeBuilder qw ( 
    CalculateAvataxChecksum
);

use Test::More;

use strict;

sub test {
    my $self = shift;

    $self->test_Calculate_Successful();

    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_Calculate_Successful {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $OrderExtractor = $self->_buildOrderExtractor();

    my $Checksum = CalculateAvataxChecksum( $OrderExtractor );
    my $ExpectedHashCode = '58da477e2444b2577ac6b8cf3417479e';
    is($Checksum, $ExpectedHashCode, "$SubTestName - Test if the checksum was calculated right");

}
#----------------------------------------------------------------------------------------
#   Helpers
#----------------------------------------------------------------------------------------
sub _createCustomerAddress {
    my $self = shift;

    my $CustomerAddres = DE_EPAGES::AvaTax::API::DTO::BaseAddress->new();
    $CustomerAddres->Line2('Dummy street 123');
    $CustomerAddres->Country('US');
    $CustomerAddres->PostalCode('55555');
    $CustomerAddres->AddressCode('Dest');
    $CustomerAddres->City('Dummy city');
    $CustomerAddres->Region('RE');
    $CustomerAddres->Line1('Test Customer');
    return $CustomerAddres;
}

sub _createContainerAttributes {
    my $self = shift;
    my $hContainerAttributes = {
        'LanguageID' => '2',
        'CurrencyID' => 'USD',
        'TaxModel' => '0',
    };
    return $hContainerAttributes;
}

sub _createLine {
    my $self = shift;
    my ($Amount, $Qty, $ItemCode, $No, $TaxCode, $Description) = @_;

    my $Line = DE_EPAGES::AvaTax::API::DTO::Line->new();
    $Line->Amount($Amount);
    $Line->Qty($Qty);
    $Line->ItemCode($ItemCode);
    $Line->No($No);
    $Line->TaxCode($TaxCode);
    $Line->Description($Description);
    return $Line;
}

sub _buildLine {
    my $self = shift;
    return $self->_createLine('60', '1', 'test_product_code', '1', 'NT', 'ProductName');
}
sub _buildShipping {
    my $self = shift;
    return $self->_createLine('5.88', '1', 'delivery', '2', 'P00000', 'StandardDelivery');
}


sub _buildOrderExtractor {
    my $self = shift;

    my $hAttributes = {
        'getCustomerAddress' => $self->_createCustomerAddress(),
        'getContainerAttributes' => $self->_createContainerAttributes(),
        'getDiscountValue' => '0',
        'getLines' => [$self->_buildLine(), $self->_buildShipping()],
    };
    my $OrderExtractor = $self->_createOrderExtractor($hAttributes);

    return $OrderExtractor;
}

sub _createOrderExtractor {
    my $self = shift;
    my ($hAttributes) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::AvaTax::API::OrderExtractor', [] );
    $Mockify->addMockWithReturnValue('getCustomerAddress', $hAttributes->{'getCustomerAddress'});
    $Mockify->addMockWithReturnValue('getContainerAttributes', $hAttributes->{'getContainerAttributes'});
    $Mockify->addMockWithReturnValue('getDiscountValue', $hAttributes->{'getDiscountValue'});
    $Mockify->addMockWithReturnValue('getLines', $hAttributes->{'getLines'});

    return $Mockify->getMockObject();
}



## use critic (ProhibitLongLines)
__PACKAGE__->runScript();