#========================================================================================
# package      DE_EPAGES::AvaTax::t::unit::OnAfterUpdateCustomerOrder
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::AvaTax::API::OnAfterUpdateCustomerOrder
#========================================================================================
package DE_EPAGES::AvaTax::t::unit::OnAfterUpdateCustomerOrder;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw ( WasCalled );
use DE_EPAGES::Core::API::Error qw ( Error );
use Test::Exception;
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_NotSuccessfulExtraction();
    $self->test_NotAbleToSendRequest();
    $self->test_ErrorCase();

    $self->test_SetOrderToPaid();
    $self->test_OrderPaid_And_HasChanged();
    $self->test_OrderPaid_And_SetToReturn();
    $self->test_OrderPaid_And_SetToCancelled();
    $self->test_SetOrderToPaid_WithoutLineItemAvatax();

    $self->test_OrderPaid_And_SetToPaid();

    done_testing();
}

#----------------------------------------------------------------------------------------
sub test_NotSuccessfulExtraction {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $OrderExtractor = $self->_buildOrderExtractor_NotSuccessfulExtraction();
    my $OnAfterUpdateCustomerOrder = $self->_buildOnAfterUpdateCustomerOrder_NotAbleToSendRequest($OrderExtractor);

    my $Order = $self->_createOrder();
    my $Result = $OnAfterUpdateCustomerOrder->do($Order, {});

    is($Result, 0, "$SubTestName - tests if the OrderExtractor was not successful")
}
#----------------------------------------------------------------------------------------
sub test_NotAbleToSendRequest {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $OrderExtractor = $self->_buildOrderExtractor_NotAbleToSendRequest();
    my $OnAfterUpdateCustomerOrder = $self->_buildOnAfterUpdateCustomerOrder_NotAbleToSendRequest($OrderExtractor);

    my $Order = $self->_createOrder();
    my $Result = $OnAfterUpdateCustomerOrder->do($Order, {});

    is($Result, 0, "$SubTestName - tests if not able to send request");
    is(WasCalled($OnAfterUpdateCustomerOrder , '_sendRequests'), 0, "$SubTestName - tests if _sendRequests was not called.");
}
#----------------------------------------------------------------------------------------
sub test_ErrorCase {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $OrderExtractor = undef;
    my $OnAfterUpdateCustomerOrder = $self->_buildOnAfterUpdateCustomerOrder_NotAbleToSendRequest($OrderExtractor);

    my $Order = $self->_createOrder();
    my $Result = $OnAfterUpdateCustomerOrder->do($Order, {});

    is($Result, 0, "$SubTestName - tests if the error is catched");
    ok( WasCalled($OnAfterUpdateCustomerOrder , '_LogError'), "$SubTestName - tests if _LogError was called.");
}
#----------------------------------------------------------------------------------------
sub test_SetOrderToPaid {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $HasChanges = 0;
    my $OrderExtractor = $self->_buildOrderExtractor_OrderNotPaid_WithoutDocCode($HasChanges);
    my $OnAfterUpdateCustomerOrder = $self->_buildOnAfterUpdateCustomerOrder($OrderExtractor);

    my $Order = $self->_createOrder();
    my $hChangedValues = {
        'PaidOn' => '1',
    };

    my $Result = $OnAfterUpdateCustomerOrder->do($Order, $hChangedValues);

    ok($Result, "$SubTestName - tests if the sales invoice was sent to avatax");
    ok( WasCalled($OnAfterUpdateCustomerOrder , '_createSalesInvoiceSender'),"$SubTestName - tests if _sendGetTaxRequest was called.");
    is(WasCalled($OnAfterUpdateCustomerOrder , '_createAdjustTaxSender'), 0, "$SubTestName - tests if _createAdjustTaxSender was not called.");
    is(WasCalled($OnAfterUpdateCustomerOrder , '_createCancelTaxSender'), 0, "$SubTestName - tests if _createCancelTaxSender was not called.");
}
#----------------------------------------------------------------------------------------
sub test_OrderPaid_And_HasChanged {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $HasChanges = 1;
    my $OrderExtractor = $self->_buildOrderExtractor_OrderPaid_WithDocCode($HasChanges);
    my $OnAfterUpdateCustomerOrder = $self->_buildOnAfterUpdateCustomerOrder($OrderExtractor);

    my $Order = $self->_createOrder();
    my $hChangedValues = {
        'PaidOn' => undef,
    };

    my $Result = $OnAfterUpdateCustomerOrder->do($Order, $hChangedValues);

    ok( $Result, "$SubTestName - tests if the sales invoice was sent to avatax");
    is( WasCalled($OnAfterUpdateCustomerOrder , '_createSalesInvoiceSender'), 0, "$SubTestName - tests if _createSalesInvoiceSender was not called.");
    ok( WasCalled($OnAfterUpdateCustomerOrder , '_createAdjustTaxSender'),"$SubTestName - tests if _sendAdjustTaxRequest was called.");
    is( WasCalled($OnAfterUpdateCustomerOrder , '_createCancelTaxSender'), 0, "$SubTestName - tests if _createCancelTaxSender was not called.");
}
#----------------------------------------------------------------------------------------
sub test_OrderPaid_And_SetToReturn {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $HasChanges = 0;
    my $OrderExtractor = $self->_buildOrderExtractor_OrderPaid_WithDocCode($HasChanges);
    my $OnAfterUpdateCustomerOrder = $self->_buildOnAfterUpdateCustomerOrder($OrderExtractor);

    my $Order = $self->_createOrder();
    my $hChangedValues = {
        'ReturnedOn' => '1',
    };

    my $Result = $OnAfterUpdateCustomerOrder->do($Order, $hChangedValues);

    ok( $Result, "$SubTestName - tests if the cancelTax was called.");
    is( WasCalled($OnAfterUpdateCustomerOrder , '_createSalesInvoiceSender'), 0, "$SubTestName - tests if _createSalesInvoiceSender was not called.");
    is( WasCalled($OnAfterUpdateCustomerOrder , '_createAdjustTaxSender'), 0, "$SubTestName - tests if _sendAdjustTaxRequest was not called.");
    ok( WasCalled($OnAfterUpdateCustomerOrder , '_createCancelTaxSender'), "$SubTestName - tests if _createCancelTaxSender was called.");
}
#----------------------------------------------------------------------------------------
sub test_OrderPaid_And_SetToCancelled {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $HasChanges = 0;
    my $OrderExtractor = $self->_buildOrderExtractor_OrderPaid_WithDocCode($HasChanges);
    my $OnAfterUpdateCustomerOrder = $self->_buildOnAfterUpdateCustomerOrder($OrderExtractor);

    my $Order = $self->_createOrder();
    my $hChangedValues = {
        'CancelledOn' => '1',
    };

    my $Result = $OnAfterUpdateCustomerOrder->do($Order, $hChangedValues);

    ok( $Result, "$SubTestName - tests if the cancelTax was called.");
    is( WasCalled($OnAfterUpdateCustomerOrder , '_createSalesInvoiceSender'), 0, "$SubTestName - tests if _createSalesInvoiceSender was not called.");
    is( WasCalled($OnAfterUpdateCustomerOrder , '_createAdjustTaxSender'), 0, "$SubTestName - tests if _sendAdjustTaxRequest was not called.");
    ok( WasCalled($OnAfterUpdateCustomerOrder , '_createCancelTaxSender'), "$SubTestName - tests if _createCancelTaxSender was called.");

}
#----------------------------------------------------------------------------------------
sub test_OrderPaid_And_SetToPaid {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $HasChanges = 0;
    my $OrderExtractor = $self->_buildOrderExtractor_OrderPaid_WithDocCode($HasChanges);
    my $OnAfterUpdateCustomerOrder = $self->_buildOnAfterUpdateCustomerOrder($OrderExtractor);

    my $Order = $self->_createOrder();
    my $hChangedValues = {
        'PaidOn' => '1',
    };

    my $Result = $OnAfterUpdateCustomerOrder->do($Order, $hChangedValues);
    is( $Result, 0, "$SubTestName - tests if nothing was called.");
    is( WasCalled($OnAfterUpdateCustomerOrder , '_createSalesInvoiceSender'), 0, "$SubTestName - tests if _createSalesInvoiceSender was not called.");
    is( WasCalled($OnAfterUpdateCustomerOrder , '_createAdjustTaxSender'), 0, "$SubTestName - tests if _sendAdjustTaxRequest was not called.");
    is( WasCalled($OnAfterUpdateCustomerOrder , '_createCancelTaxSender'), 0, "$SubTestName - tests if _createCancelTaxSender was not called.");
}

sub test_SetOrderToPaid_WithoutLineItemAvatax {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $HasChanges = 0;
    my $OrderExtractor = $self->_buildOrderExtractor_OrderNotPaid_WithoutLineItemAvatax($HasChanges);
    my $OnAfterUpdateCustomerOrder = $self->_buildOnAfterUpdateCustomerOrder($OrderExtractor);

    my $Order = $self->_createOrder();
    my $hChangedValues = {
        'PaidOn' => '1',
    };

    my $Result = $OnAfterUpdateCustomerOrder->do($Order, $hChangedValues);

    is( $Result, 0, "$SubTestName - tests if nothing was called.");
    is( WasCalled($OnAfterUpdateCustomerOrder , '_createSalesInvoiceSender'), 0, "$SubTestName - tests if _createSalesInvoiceSender was not called.");
    is( WasCalled($OnAfterUpdateCustomerOrder , '_createAdjustTaxSender'), 0, "$SubTestName - tests if _sendAdjustTaxRequest was not called.");
    is( WasCalled($OnAfterUpdateCustomerOrder , '_createCancelTaxSender'), 0, "$SubTestName - tests if _createCancelTaxSender was not called.");
}
#========================================================================================
# Helpers
#========================================================================================
sub _buildOnAfterUpdateCustomerOrder_NotAbleToSendRequest{
    my $self = shift;
    my ($OrderExtractor) = @_;

    my $Mockify = $self->_createMockedOnAfterUpdateCustomerOrder($OrderExtractor);
    $Mockify->addMock('_sendRequests', sub {Error('ThisMethodShouldBeCalled')}); #forbidden call

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _buildOnAfterUpdateCustomerOrder{
    my $self = shift;
    my ($OrderExtractor) = @_;

    my $Mockify = $self->_createMockedOnAfterUpdateCustomerOrder($OrderExtractor);
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _buildOrderExtractor_OrderPaid_WithDocCode {
    my $self = shift;
    my ($HasChanged) = @_;

    my $WasSuccessFullExtraction = 1;
    my $LineItemContainer = $self->_createLineItemContainer({ 'AvaTax_DocCode' => 'Test-Doc-Code-1337' });
    my $TaxMethod = $self->_createTaxMethod({ 'IsTaxDocumentTransmissionActivated' => 1, });
    my $Shop = $self->_createShop($TaxMethod);
    my $isPaid = 1;

    return $self->_createOrderExtractor($Shop, $LineItemContainer, $WasSuccessFullExtraction, $HasChanged, $isPaid);
}
#----------------------------------------------------------------------------------------
sub _buildOrderExtractor_OrderNotPaid_WithoutDocCode {
    my $self = shift;
    my ($HasChanged) = @_;

    my $WasSuccessFullExtraction = 1;
    my $LineItemContainer = $self->_createLineItemContainer({ 'AvaTax_DocCode' => undef });
    my $TaxMethod = $self->_createTaxMethod({ 'IsTaxDocumentTransmissionActivated' => 1, });
    my $Shop = $self->_createShop($TaxMethod);
    my $isPaid = 0;

    return $self->_createOrderExtractor($Shop, $LineItemContainer, $WasSuccessFullExtraction, $HasChanged, $isPaid);
}
#----------------------------------------------------------------------------------------
sub _buildOrderExtractor_NotAbleToSendRequest {
    my $self = shift;

    my $WasSuccessFullExtraction = 1;
    my $LineItemContainer = 'Not Relevant for this tests';
    my $TaxMethod = $self->_createTaxMethod({ 'IsTaxDocumentTransmissionActivated' => 0 });
    my $Shop = $self->_createShop($TaxMethod);
    my $HasChanged = 0;
    my $isPaid = 0;

    return $self->_createOrderExtractor($Shop, $LineItemContainer, $WasSuccessFullExtraction, $HasChanged, $isPaid);
}
#----------------------------------------------------------------------------------------
sub _buildOrderExtractor_NotSuccessfulExtraction{
    my $self = shift;

    my $WasSuccessFullExtraction = 0;
    my $Shop = 'Not relevant for this test';
    my $LineItemContainer = 'not relevant for this test';
    my $HasChanged = 0;
    my $isPaid = 0;

    return $self->_createOrderExtractor($Shop, $LineItemContainer, $WasSuccessFullExtraction, $HasChanged, $isPaid);
}
#---------------------------------------------------------------------------
#  Helpers mockify
#---------------------------------------------------------------------------
sub _createMockedOnAfterUpdateCustomerOrder {
    my $self = shift;
    my ($OrderExtractor) = @_;

    my $MockifyObject = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::AvaTax::API::OnAfterUpdateCustomerOrder',
        []
    );
    $MockifyObject->addMockWithReturnValue('_createOrderExtractor', $OrderExtractor);
    $MockifyObject->addMockWithReturnValue('_createSalesInvoiceSender',$self->_createSalesInvoiceSender());
    $MockifyObject->addMockWithReturnValue('_createAdjustTaxSender',$self->_createAdjustTaxSender());
    $MockifyObject->addMockWithReturnValue('_createCancelTaxSender',$self->_createCancelTaxSender());
    $MockifyObject->addMockWithReturnValueAndParameterCheck('_LogError', undef, ['object']);

    return $MockifyObject;
}
#----------------------------------------------------------------------------------------
sub _buildOrderExtractor_OrderNotPaid_WithoutLineItemAvatax {
    my $self = shift;
    my ($HasChanged) = @_;

    my $WasSuccessFullExtraction = 1;
    my $LineItemContainer = undef;
    my $TaxMethod = $self->_createTaxMethod({ 'IsTaxDocumentTransmissionActivated' => 1, });
    my $Shop = $self->_createShop($TaxMethod);
    my $isPaid = 0;

    return $self->_createOrderExtractor($Shop, $LineItemContainer, $WasSuccessFullExtraction, $HasChanged, $isPaid);
}
#----------------------------------------------------------------------------------------
sub _createOrderExtractor {
    my $self = shift;
    my ($Shop, $LineItemContainer, $WasSuccessFullExtraction, $HasChanged, $isPaid) = @_;

    my $MockifyObject = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::AvaTax::API::OrderExtractor',
        []
    );
    $MockifyObject->addMockWithReturnValueAndParameterCheck('extractDataFrom', $WasSuccessFullExtraction,['object']);
    $MockifyObject->addMockWithReturnValue('getShop', $Shop);
    $MockifyObject->addMockWithReturnValue('hasChanged', $HasChanged);
    $MockifyObject->addMockWithReturnValue('isPaid', $isPaid);
    $MockifyObject->addMockWithReturnValue('updateHash',undef);
    $MockifyObject->addMockWithReturnValue('getLineItemContainer', $LineItemContainer);

    return $MockifyObject->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createSalesInvoiceSender {
    my $self = shift;
    my $MockifyObject = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::AvaTax::API::SalesInvoiceSender',
        []
    );
    $MockifyObject->addMockWithReturnValueAndParameterCheck('sendSalesInvoice', undef, [{'object' => 'DE_EPAGES::AvaTax::API::OrderExtractor'}]);
    return $MockifyObject->getMockObject();
}
#---------------------------------------------------------------------------
sub _createAdjustTaxSender {
    my $self = shift;
    my $MockifyObject = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::AvaTax::API::AdjustTaxSender',
        []
    );
    $MockifyObject->addMockWithReturnValueAndParameterCheck('sendAdjustTax', undef, [{'object' => 'DE_EPAGES::AvaTax::API::OrderExtractor'}]);
    return $MockifyObject->getMockObject();
}
#---------------------------------------------------------------------------
sub _createCancelTaxSender {
    my $self = shift;
    my $MockifyObject = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::AvaTax::API::CancelTaxSender',
        []
    );
    $MockifyObject->addMockWithReturnValueAndParameterCheck('sendCancelTax', undef, [{'object' => 'DE_EPAGES::AvaTax::API::OrderExtractor'}]);
    return $MockifyObject->getMockObject();
}
#---------------------------------------------------------------------------
#  Helpers ePages objects
#---------------------------------------------------------------------------
sub _createOrder {
    my $self = shift;
    my $MockedOrder = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::CustomerOrder', {});
    return $MockedOrder;
}
#---------------------------------------------------------------------------
sub _createShop {
    my $self = shift;
    my ($TaxMethod) = @_;
    my $hAttributes = {
        'TaxMethod' => $TaxMethod,
    };
    my $MockedObject = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
    return $MockedObject;
}
#---------------------------------------------------------------------------
sub _createTaxMethod {
    my $self = shift;
    my ($hAttributes) = @_;
    my $MockedObject = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::AvaTax::API::Object::TaxMethodAvaTax', $hAttributes);
    return $MockedObject;
}
#---------------------------------------------------------------------------
sub _createLineItemContainer {
    my $self = shift;
    my ($hAttributes) = @_;
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes);
}

__PACKAGE__->runScript();
