#========================================================================================
# package      DE_EPAGES::AvaTax::t::unit::ResponseParser
# base         DE_EPAGES::Core::API::Test
#----------------------------------------------------------------------------------------
# description  unit test for ResponseParser
#========================================================================================
package DE_EPAGES::AvaTax::t::unit::ResponseParser;
use base DE_EPAGES::Core::API::Test;
use Test::More;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::AvaTax::API::Constants qw (
    AVATAX_GETTAX_SUCCESS
    AVATAX_ADJUSTTAX_SUCCESS
    AVATAX_CANCELTAX_SUCCESS
    AVATAX_WEBSERVICE_ERROR
);
use DE_EPAGES::AvaTax::API::ResponseParser;
use strict;

sub test {
    my $self = shift;
    $self->test_SuccessfulResponseGetTax();
    $self->test_SuccessfulResponseAdjustTax();
    $self->test_SuccessfulResponseCancelTax();
    $self->test_ErrorResponse();
    $self->test_ErrorSOAP();
    done_testing();
}

sub test_SuccessfulResponseGetTax {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ResponseParser = $self->_createResponseParser($self->_createGetTaxResult_Success());
    is($ResponseParser->getStatusGetTax(), AVATAX_GETTAX_SUCCESS, "$SubTestName - Test if status is Success");
    is($ResponseParser->getStatusMessage(), '', "$SubTestName - Test if message is empty");;
}

sub test_SuccessfulResponseAdjustTax {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ResponseParser = $self->_createResponseParser($self->_createGetTaxResult_Success());
    is($ResponseParser->getStatusAdjustTax(), AVATAX_ADJUSTTAX_SUCCESS, "$SubTestName - Test if status is Success");
    is($ResponseParser->getStatusMessage(), '', "$SubTestName - Test if message is empty");;
}

sub test_SuccessfulResponseCancelTax {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ResponseParser = $self->_createResponseParser($self->_createGetTaxResult_Success());
    is($ResponseParser->getStatusCancelTax(), AVATAX_CANCELTAX_SUCCESS, "$SubTestName - Test if status is Success");
    is($ResponseParser->getStatusMessage(), '', "$SubTestName - Test if message is empty");;
}

sub test_ErrorResponse {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ResponseParser = $self->_createResponseParser($self->_createGetTaxResult_ErrorResponse());
    is($ResponseParser->getStatusGetTax(), AVATAX_WEBSERVICE_ERROR, "$SubTestName - Test if status is Error");
    is($ResponseParser->getStatusMessage(), 'Error1 ', "$SubTestName - Test if message is Error1");;
}

sub test_ErrorSOAP {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ResponseParser = $self->_createResponseParser($self->_createGetTaxResult_ErrorSOAP());
    is($ResponseParser->getStatusGetTax(), AVATAX_WEBSERVICE_ERROR, "$SubTestName - Test if status is Error");
    is($ResponseParser->getStatusMessage(), 'Something wrong with the SOAP request', "$SubTestName - Test if message is from soap error");;
}

sub _createResponseParser {
    my $self = shift;
    my ($Response) = @_;
    return DE_EPAGES::AvaTax::API::ResponseParser->new($Response);
}

sub _createGetTaxResult_Success {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::AvaTax::API::DTO::GetTaxResult',
        $aParameterList
    );
    $Mockify->addMockWithReturnValue('ResultCode', 'Success');

    return $Mockify->getMockObject();
}

sub _createGetTaxResult_ErrorResponse {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::AvaTax::API::DTO::GetTaxResult',
        $aParameterList
    );
    $Mockify->addMockWithReturnValue('ResultCode', 'Error');
    $Mockify->addMockWithReturnValue('Messages', $self->_createMessages());

    return $Mockify->getMockObject();
}

sub _createGetTaxResult_ErrorSOAP {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::AvaTax::API::DTO::SOAPFault',
        $aParameterList
    );
    $Mockify->addMockWithReturnValue('faultstring', 'Something wrong with the SOAP request');

    return $Mockify->getMockObject();
}

sub _createMessages {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::AvaTax::API::DTO::Message',
        $aParameterList
    );
    $Mockify->addMockWithReturnValue('Summary', 'Error1');

    my $aMessages = [$Mockify->getMockObject()];
    return $aMessages;
}
__PACKAGE__->runScript();