#========================================================================================
# package      DE_EPAGES::AvaTax::t::unit::SalesInvoiceSender
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  test DE_EPAGES::AvaTax::API::SalesInvoiceSender
#========================================================================================
package DE_EPAGES::AvaTax::t::unit::SalesInvoiceSender;
use base DE_EPAGES::Core::API::Test;
use Test::More;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::MockObject;
use DE_EPAGES::Core::API::TestParameter qw (TestObject);
use DE_EPAGES::Test::API::TestTools qw (TestExistsMethod);
use DE_EPAGES::Core::API::PerlTools qw ( UsePackage );
use DE_EPAGES::AvaTax::API::CancelTaxSender;
use DE_EPAGES::AvaTax::API::DTO::GetTaxRequest;
use DE_EPAGES::AvaTax::API::Constants qw (AVATAX_DOCUMENTTYPE_SalesInvoice AVATAX_GETTAX_SUCCESS);

use strict;

sub test {
    my $self = shift;
    $self->test_sendGetTaxRequestWhenCommitted();
    $self->test_sendGetTaxRequestWhenCommitted_WithoutTaxExemption();
    $self->test_sendGetTaxRequestWhenUncommitted();
    done_testing();
}

sub test_sendGetTaxRequestWhenCommitted {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $Response = $self->_buildCommittedResponse();
    my $RemoteConnector = $self->_createRemoteConnector($Response);
    my $SalesInvoiceSender = $self->_buildSalesInvoiceSender_StatusCommitted($RemoteConnector);

    my $LineItemContainer = $self->_createLineItemContainer();
    my $OrderExtractor = $self->_buildOrderExtractor_WithAvataxExemption($LineItemContainer);
    $SalesInvoiceSender->sendSalesInvoice( $OrderExtractor );

    my $GetTaxRequest = $self->{'param_getTax'}; # get the parameter from getTax call
    is($GetTaxRequest->DocType(),AVATAX_DOCUMENTTYPE_SalesInvoice,"$SubTestName - test DocType return value");
    is($GetTaxRequest->Commit(),'true',"$SubTestName - test commit return value");
    is($GetTaxRequest->CustomerUsageType(),'Test-Tax-Exemption',"$SubTestName - test CustomerUsageType value is set");

    is($LineItemContainer->get('AvaTax_StatusResponse'), AVATAX_GETTAX_SUCCESS, "$SubTestName - tests if Status is set correctly");
    is($LineItemContainer->get('AvaTax_StatusMessage'), 'message', "$SubTestName - tests if StatusMesssage is set correctly");
    is($LineItemContainer->get('AvaTax_TimeStamp'), 'now', "$SubTestName - tests if TimeStamp is set correctly");
    is($LineItemContainer->get('AvaTax_DocCode'), 'Test-Doc-Code-123', "$SubTestName - tests if DocCode is set correctly");

    $self->{'param_getTax'} = undef; #rollback
}

sub test_sendGetTaxRequestWhenCommitted_WithoutTaxExemption {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $Response = $self->_buildCommittedResponse();
    my $RemoteConnector = $self->_createRemoteConnector($Response);
    my $SalesInvoiceSender = $self->_buildSalesInvoiceSender_StatusCommitted($RemoteConnector);

    my $LineItemContainer = $self->_createLineItemContainer();
    my $OrderExtractor = $self->_buildOrderExtractor_WithoutAvataxExemption($LineItemContainer);
    $SalesInvoiceSender->sendSalesInvoice( $OrderExtractor );

    my $GetTaxRequest = $self->{'param_getTax'}; # get the parameter from getTax call
    is($GetTaxRequest->DocType(),AVATAX_DOCUMENTTYPE_SalesInvoice,"$SubTestName - test DocType return value");
    is($GetTaxRequest->Commit(),'true',"$SubTestName - test commit return value");
    is($GetTaxRequest->CustomerUsageType(), undef,"$SubTestName - test CustomerUsageType value is undef");

    is($LineItemContainer->get('AvaTax_StatusResponse'), AVATAX_GETTAX_SUCCESS, "$SubTestName - tests if Status is set correctly");
    is($LineItemContainer->get('AvaTax_StatusMessage'), 'message', "$SubTestName - tests if StatusMesssage is set correctly");
    is($LineItemContainer->get('AvaTax_TimeStamp'), 'now', "$SubTestName - tests if TimeStamp is set correctly");
    is($LineItemContainer->get('AvaTax_DocCode'), 'Test-Doc-Code-123', "$SubTestName - tests if DocCode is set correctly");

    $self->{'param_getTax'} = undef; #rollback
}

sub test_sendGetTaxRequestWhenUncommitted {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $Response = $self->_buildUncommittedResponse();
    my $RemoteConnector = $self->_createRemoteConnector($Response);
    my $SalesInvoiceSender = $self->_buildSalesInvoiceSender_StatusUncommitted($RemoteConnector);

    my $LineItemContainer = $self->_createLineItemContainer();
    my $OrderExtractor = $self->_buildOrderExtractor_WithAvataxExemption($LineItemContainer);
    $SalesInvoiceSender->sendSalesInvoice( $OrderExtractor );

    my $GetTaxRequest = $self->{'param_getTax'}; # get the parameter from getTax call
    is($GetTaxRequest->DocType(),AVATAX_DOCUMENTTYPE_SalesInvoice,"$SubTestName - test DocType return value");
    is($GetTaxRequest->Commit(),'true',"$SubTestName - test commit return value");
    is($GetTaxRequest->CustomerUsageType(),'Test-Tax-Exemption',"$SubTestName - test CustomerUsageType value is set");

    is($LineItemContainer->get('AvaTax_StatusResponse'), AVATAX_GETTAX_SUCCESS, "$SubTestName - tests if Status is set correctly");
    is($LineItemContainer->get('AvaTax_StatusMessage'), 'message', "$SubTestName - tests if StatusMesssage is set correctly");
    is($LineItemContainer->get('AvaTax_TimeStamp'), 'now', "$SubTestName - tests if TimeStamp is set correctly");
    is($LineItemContainer->get('AvaTax_DocCode'), 'Test-Doc-Code-123', "$SubTestName - tests if DocCode is set correctly");

    $self->{'param_getTax'} = undef; #rollback
}

#========================================================================================
# Helpers
#========================================================================================
sub _buildSalesInvoiceSender_StatusCommitted {
    my $self = shift;

    my $Response = $self->_buildCommittedResponse();
    my $RemoteConnector = $self->_createRemoteConnector($Response);
    my $hSetupSalesInvoiceSender = {
        'RemoteConnector' => $RemoteConnector,
        'LogMessage' => "The order MyOrder has been commited!\n",
    };

    return $self->_createSalesInvoiceSender($hSetupSalesInvoiceSender);
}

sub _buildSalesInvoiceSender_StatusUncommitted {
    my $self = shift;

    my $Response = $self->_buildUncommittedResponse();
    my $RemoteConnector = $self->_createRemoteConnector($Response);
    my $hSetupSalesInvoiceSender = {
        'RemoteConnector' => $RemoteConnector,
        'LogMessage' => "The order MyOrder has not been commited!\n",
    };

    return $self->_createSalesInvoiceSender($hSetupSalesInvoiceSender);
}

sub _createSalesInvoiceSender {
    my $self = shift;
    my ($hSetupMockedReturnInvoiceSender) = @_;

    my $MockifyObject = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::AvaTax::API::SalesInvoiceSender', [] );

    my $aParamTypes = [{'object' => 'DE_EPAGES::Shop::API::Object::Shop'}];
    $MockifyObject->addMockWithReturnValueAndParameterCheck(
        '_connectToTaxService',
        $hSetupMockedReturnInvoiceSender->{'RemoteConnector'},
        $aParamTypes );
    $MockifyObject->addMockWithReturnValue(
        '_createGetTaxRequestBuilder',
        $self->_createGetTaxRequestBuilder()
    );
    my $aParameterTypes = [
        {'object' => 'DE_EPAGES::Shop::API::Object::Shop'},
        {'string' => $hSetupMockedReturnInvoiceSender->{'LogMessage'}}
    ];
    $MockifyObject->addMockWithReturnValueAndParameterCheck( '_LogInfo', undef, $aParameterTypes );
    $MockifyObject->addMockWithReturnValue('_getCurrentDateTime', 'now');
    $MockifyObject->addMockWithReturnValueAndParameterCheck('_createResponseParser', $self->_createResponseParser(), [
        {'object' => 'DE_EPAGES::AvaTax::API::DTO::GetTaxResult'}
    ]);

    return $MockifyObject->getMockObject();
}

sub _createShop {
    my $self = shift;
    my ($TaxMethod) = @_;

    my $hAttributes = {
        'TaxMethod' => $TaxMethod,
        'Alias'     => 'MyShop',
    };

    my $MockedObject = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop', $hAttributes );
    return $MockedObject;
}


sub _buildOrderExtractor_WithAvataxExemption{
    my $self = shift;
    my ($LineItemAvaTax) = @_;
    my $AvataxExemption = $self->_createAvaTaxExemption();
    return $self->_createOrderExtractor($AvataxExemption, $LineItemAvaTax);
}
sub _buildOrderExtractor_WithoutAvataxExemption{
    my $self = shift;
    my ($LineItemAvaTax) = @_;
    my $AvataxExemption = undef;
    return $self->_createOrderExtractor($AvataxExemption, $LineItemAvaTax);
}

sub _createOrderExtractor {
    my $self = shift;
    my ($AvataxExemption, $LineItemContainer) = @_;

    my $MockifyObject = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::AvaTax::API::OrderExtractor', [] );
    $MockifyObject->addMockWithReturnValue( 'getShop', $self->_createShop() );
    $MockifyObject->addMockWithReturnValue('getLineItemContainer', $LineItemContainer);
    $MockifyObject->addMockWithReturnValue( 'getAvataxTaxExemption', $AvataxExemption );
    $MockifyObject->addMockWithReturnValue( 'getOrderAlias', 'MyOrder' );

    return $MockifyObject->getMockObject();
}

sub _createLineItemContainer {
    my $self = shift;
    my $hAttributes = {
        'AvaTax_DocCode' => undef,
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes);
}

sub _createRemoteConnector {
    my $self = shift;
    my ($Response) = @_;

    UsePackage( 'DE_EPAGES::AvaTax::API::RemoteTaxSvc' );
    TestExistsMethod('DE_EPAGES::AvaTax::API::RemoteTaxSvc', 'getTax');
    my $Mock = Test::MockObject->new();
    $Mock->mock('getTax', sub {
        my $mockedself = shift;
        my ($Request) = @_;
        $self->{'param_getTax'} = $Request;
        TestObject('getTaxRequest object', $Request, 'DE_EPAGES::AvaTax::API::DTO::GetTaxRequest');
        return $Response;
    });

    return $Mock;
}

sub _createGetTaxRequestBuilder {
    my $self = shift;

    my $MockifyObject = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::AvaTax::API::GetTaxRequestBuilder', []);
    my $TaxRequest = DE_EPAGES::AvaTax::API::DTO::GetTaxRequest->new();

    $MockifyObject->addMockWithReturnValueAndParameterCheck('buildRequest', $TaxRequest, [{'object' => 'DE_EPAGES::AvaTax::API::OrderExtractor'}]);
    return $MockifyObject->getMockObject();
}

sub _buildUncommittedResponse {
    my $self = shift;

    return $self->_createResponse('Uncommitted');
}

sub _buildCommittedResponse {
    my $self = shift;

    return $self->_createResponse('Committed');
}

sub _createResponse {
    my $self = shift;
    my ($DocStatus) = @_;

    my $GetTaxResult = DE_EPAGES::AvaTax::API::DTO::GetTaxResult->new();
    $GetTaxResult->DocStatus($DocStatus);
    $GetTaxResult->DocCode('Test-Doc-Code-123');

    return $GetTaxResult;
}

sub _createAvaTaxExemption {
    my $self = shift;

    my $hAtributes = {'Alias' => 'Test-Tax-Exemption'};
    my $MockedObject = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::PreDefAttribute', $hAtributes );

    return $MockedObject;
}

sub _createResponseParser {
    my $self = shift;
    my $MockifyObject = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::AvaTax::API::ResponseParser',
        []
    );
    $MockifyObject->addMockWithReturnValue('getStatusGetTax', AVATAX_GETTAX_SUCCESS);
    $MockifyObject->addMockWithReturnValue('getStatusMessage', 'message');
    return $MockifyObject->getMockObject();
}

__PACKAGE__->runScript();
