#!perl -w
use Test::More tests => 12;
use strict;
use Data::Dumper;
use DateTime;
use File::Find qw( find );
use File::Temp qw( tempdir );

use DE_EPAGES::Core::API::File qw ( RunOnDirectory CopyDirectory DeleteDirectory GetFileContentLines GetFileNames );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::SystemCommand qw( SystemCommand );

sub Main {
    local $| = 1;

    my $Perl = $^O eq 'linux' ? $ENV{PERL} : $^X;
    my $MySQLArchiveDumps = "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/Database/Scripts/MySQLArchiveDumps.pl";
    ##no critic qw(ProhibitMagicNumbers)
    my $BaseDate = 11240; # (2000-10-10 00:00 GMT)
    my $Today = int(time() / (60 * 60 * 24)); # (today 00:00 GMT)
    my $DateShift = $Today - $BaseDate; # in days
    ##use critic
    my $TempDir = tempdir();

    foreach my $Zipper ('', 'touch', 'bzip2 -9') {
        SKIP: {
            my $Exe = $Zipper ? $Zipper : 'gzip';
            $Exe =~ s, .*,,;
            skip "Zipper not installed on $^O: $Exe", 4 if $Exe eq 'bzip2' and $^O eq 'MSWin32';

            foreach my $Days ('', 0, 5, 300) { ##no critic qw(ProhibitMagicNumbers)
                CopyDirectory('MySQLArchiveDumps/base', $TempDir);
                _TouchDirectory($TempDir);

                my $Arguments = "-dumpdir \"$TempDir\"";
                $Arguments .= " -zipper \"$Zipper\"" if $Zipper =~ /\S/;
                $Arguments .= sprintf(' -delold %d', $DateShift + $Days) if $Days =~ /\S/;

                SystemCommand("$Perl $MySQLArchiveDumps $Arguments -nooutput");

                my @ExpectedFiles = map { chomp; $_; } @{
                    GetFileContentLines("MySQLArchiveDumps/MySQLArchiveDumps.$Exe.$Days.expected")
                };

                my @FoundFiles = sort
                    grep { !( m,/CVS/, || m,CVS, ) }
                    map { m,^\./(.*), ? $1 : $_ }
                    RunOnDirectory($TempDir, sub { @{GetFileNames(undef, {'recursive' => 1})} });

                is_deeply(\@FoundFiles, \@ExpectedFiles, "perl MySQLArchiveDumps.pl $Arguments")
                    or diag( Dumper(\@FoundFiles) .'~~'. Dumper(\@ExpectedFiles) );
                DeleteDirectory($TempDir, {'recursive' => 1});
            }
        }
    }
}

sub _TouchDirectory {
    my ($Directory) = @_;
    # set file time derived from file name
    find( sub {
        return unless $File::Find::name =~ /_(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})/;
        my $Time = DateTime->new(
            year => $1, month  => $2, day => $3,
            hour => $4, minute => $5, time_zone => 'Europe/London')->epoch; #set to GMT 
        utime($Time, $Time, $File::Find::name);
          },
          $Directory
        );
}

RunScript(Sub=>\&Main);
1;
