#========================================================================================
# package      DE_EPAGES::FedEx::unit::t::FedExShippingProductParser
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for FedExShippingProductParser
#========================================================================================
package DE_EPAGES::FedEx::unit::t::FedExShippingProductParser;
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::FedEx::API::FedExShippingProductParser;
use SOAP::Lite;
use DE_EPAGES::Core::API::File qw ( GetFileContent );
use Test::More;
use DE_EPAGES::Test::Mock::Mockify qw (GetParametersFromMockifyCall);
use strict;

sub test {
    my $self = shift;

    $self->test_multipleProducts();
    $self->test_singleProduct();
    $self->test_singleProduct_MultipleRatedShippingDetails();
    $self->test_No_RatedShipmentDetails();
    $self->test_No_RateReplyDetail();
    $self->test_Not_Successful();
    $self->test_No_HighestSeverity();
    $self->test_Unknown_ProductType();

    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_multipleProducts {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Xml = $self->_loadFile('Successful_MultipleProducts.xml');
    my $SoapResponse = SOAP::Deserializer->deserialize($Xml);
    my $FedExShippingProductParser = $self->_createFedExShippingProductParser();

    my $hParsedResponse = $FedExShippingProductParser->parse($SoapResponse);

    is_deeply(
        $hParsedResponse->{'data'}, [
          {
            'Currency' => 'EUR',
            'ShippingFulfillmentProduct' => 'FEDEX_2_DAY',
            'ShippingFulfillmentProductText' => 'FedEx 2 Day',
            'Price' => '110.1'
          },
          {
            'Currency' => 'USD',
            'ShippingFulfillmentProduct' => 'FIRST_OVERNIGHT',
            'ShippingFulfillmentProductText' => 'First Overnight',
            'Price' => '220.2'
          }
        ],
        "$SubTestName - Test for multipleProducts");
}

#----------------------------------------------------------------------------------------
sub test_singleProduct {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Xml = $self->_loadFile('Successful_SingleProduct.xml');
    my $SoapResponse = SOAP::Deserializer->deserialize($Xml);
    my $FedExShippingProductParser = $self->_createFedExShippingProductParser();

    my $hParsedResponse = $FedExShippingProductParser->parse($SoapResponse);

    is_deeply(
        $hParsedResponse->{'data'}, [
          {
            'Currency' => 'EUR',
            'ShippingFulfillmentProduct' => 'FEDEX_2_DAY',
            'ShippingFulfillmentProductText' => 'FedEx 2 Day',
            'Price' => '110.1'
          }
        ],
        "$SubTestName - Test for single Product");
}
#----------------------------------------------------------------------------------------
sub test_singleProduct_MultipleRatedShippingDetails {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Xml = $self->_loadFile('Successful_SingleProduct_MultipleRatedShippingDetails.xml');
    my $SoapResponse = SOAP::Deserializer->deserialize($Xml);
    my $FedExShippingProductParser = $self->_createFedExShippingProductParser();

    my $hParsedResponse = $FedExShippingProductParser->parse($SoapResponse);

    is_deeply(
        $hParsedResponse->{'data'}, [
          {
            'Currency' => 'AUS',
            'ShippingFulfillmentProduct' => 'PRIORITY_OVERNIGHT',
            'ShippingFulfillmentProductText' => 'Priority Overnight',
            'Price' => '330.3'
          }
        ],
        "$SubTestName - Test for single Product with multiple RatedShippingDetails");
}
#----------------------------------------------------------------------------------------
sub test_No_RatedShipmentDetails {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Xml = $self->_loadFile('Error_No_RatedShipmentDetails.xml');
    my $SoapResponse = SOAP::Deserializer->deserialize($Xml);
    my $FedExShippingProductParser = $self->_createFedExShippingProductParser();

    my $hParsedResponse = $FedExShippingProductParser->parse($SoapResponse);
    is_deeply( $hParsedResponse->{'status'}, {'Error' => 'Unknown Error'}, "$SubTestName - Test for no RatedShipmentDetails");
    my $aParameters_LogShippingFulfillment = GetParametersFromMockifyCall($FedExShippingProductParser, '_LogShippingFulfillment');

    is_deeply(
        $aParameters_LogShippingFulfillment, [
          'FedEx',
          'incomplete RatedShipmentDetails',
          {
            'RatedShipmentDetails' => '',
            'ServiceType' => 'FEDEX_2_DAY'
          }
        ],
        "$SubTestName - Test parameters of _LogShippingFulfillment"
    );
}

#----------------------------------------------------------------------------------------
sub test_No_RateReplyDetail {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Xml = $self->_loadFile('Error_No_RateReplyDetail.xml');
    my $SoapResponse = SOAP::Deserializer->deserialize($Xml);
    my $FedExShippingProductParser = $self->_createFedExShippingProductParser();

    my $hParsedResponse = $FedExShippingProductParser->parse($SoapResponse);

    is_deeply( $hParsedResponse->{'status'}, {'Error' => 'Unknown Error'}, "$SubTestName - Test for no RateReplyDetail");
    my $aParameters_LogShippingFulfillment = GetParametersFromMockifyCall($FedExShippingProductParser, '_LogShippingFulfillment');
    is_deeply(
        $aParameters_LogShippingFulfillment,
        [ 'FedEx', 'incomplete RatedShipmentDetails', '' ],
        "$SubTestName - Test parameters of _LogShippingFulfillment"
    );
}
#----------------------------------------------------------------------------------------
sub test_Not_Successful {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Xml = $self->_loadFile('Error_Not_Successful.xml');
    my $SoapResponse = SOAP::Deserializer->deserialize($Xml);
    my $FedExShippingProductParser = $self->_createFedExShippingProductParser();

    my $hParsedResponse = $FedExShippingProductParser->parse($SoapResponse);

    is_deeply( $hParsedResponse->{'status'}, { 'FAILURE' => 'Error reported by FedEx' }, "$SubTestName - Test for not successful case");

}
#----------------------------------------------------------------------------------------
sub test_No_HighestSeverity {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Xml = $self->_loadFile('Error_No_HighestSeverity.xml');
    my $SoapResponse = SOAP::Deserializer->deserialize($Xml);

    my $FedExShippingProductParser = $self->_createFedExShippingProductParser();
    my $hParsedResponse = $FedExShippingProductParser->parse($SoapResponse);

    is_deeply( $hParsedResponse->{'status'}, { 'Error' => 'Unknown Error', }, "$SubTestName - Test for error case");
    my $aParameters_LogShippingFulfillment = GetParametersFromMockifyCall($FedExShippingProductParser, '_LogShippingFulfillment');
    is_deeply(
        $aParameters_LogShippingFulfillment,
        [ 'FedEx', 'Incomplete Response', { 'Body' => '' } ],
        "$SubTestName - Test parameters of _LogShippingFulfillment"
    );
}

#----------------------------------------------------------------------------------------
sub test_Unknown_ProductType {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Xml = $self->_loadFile('Successful_Unknown_ProductType.xml');
    my $SoapResponse = SOAP::Deserializer->deserialize($Xml);

    my $FedExShippingProductParser = $self->_createFedExShippingProductParser();
     my $hParsedResponse = $FedExShippingProductParser->parse($SoapResponse);

    is_deeply(
            $hParsedResponse->{'data'},
            [
              {
                'Currency' => 'USD',
                'ShippingFulfillmentProduct' => 'FEDEX_FIRST_FREIGHT',
                'ShippingFulfillmentProductText' => 'FEDEX_FIRST_FREIGHT',
                'Price' => '2289.18'
              }
            ],
            "$SubTestName - Test the fallback if the FedExProduct have no translation"
    );
}

#----------------------------------------------------------------------------------------
#  helpers
#----------------------------------------------------------------------------------------
sub _createFedExShippingProductParser {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::FedEx::API::FedExShippingProductParser', [] );
    $Mockify->addMockWithReturnValueAndParameterCheck('_LogShippingFulfillment', undef, ['string','string','any']);

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _loadFile {
    my $self = shift;
    my ($XMLFile) = @_;
    my $AbsolutPath = "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/FedEx/t/unit/TestResponses/$XMLFile";
    return ${GetFileContent($AbsolutPath)};
}

__PACKAGE__->runScript();