#========================================================================================
# package      DE_EPAGES::Order::t::unit::OrderCollectionResource
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Order::API::Resource::OrderCollection
#========================================================================================
package DE_EPAGES::Order::t::unit::OrderCollectionResource;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::Mockery;
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_get();

    done_testing();
}



sub test_get {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $OrderCollectionResource = $Mockery->get('SUT');

    my $Shop = $Mockery->get('Shop');
    $Shop->set({'Site' => $Shop});
    my $ResultsPerPage = 10; ## no critic (ProhibitMagicNumbers)
    my $Page = 1;
    my $hQuery = {};

    my $hOrderCollectionData = $OrderCollectionResource->get($Shop, 'en_GB', $ResultsPerPage, $Page, $hQuery);

    is_deeply($hOrderCollectionData, $self->_createOrderData(), "$SubTestName - test if expected order data was returned");
}



sub _createMocks {
    my $self = shift;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('SUT', ['Locale', 'Class', 'PagerSearch', 'LineItemContainerData'], sub {
        my ($Locale, $Class, $PagerSearch, $LineItemContainerData) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Order::API::Resource::OrderCollection', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('_loadLocaleById', $Locale, ['string']);
        $Mockify->addMockWithReturnValueAndParameterCheck('_LoadClassByAlias', $Class, ['string']);
        $Mockify->addMockWithReturnValueAndParameterCheck('_GetSearchDefinition', undef, ['object', 'string']);
        $Mockify->addMockWithReturnValueAndParameterCheck('_createPagerSearch', $PagerSearch, ['hashref']);
        $Mockify->addMockWithReturnValueAndParameterCheck('_createLineItemContainerData', $LineItemContainerData, ['object']);
        return $Mockify->getMockObject();
    });
    ## no critic (ProhibitMagicNumbers)
    $Mockery->register('Order', ['Shop', 'Customer', 'LineItemContainer'], sub {
        my ($Shop, $Customer, $LineItemContainer) = @_;
        my $MockifyEpagesObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Order::API::Object::CustomerOrder', {
            'Alias' => 'TestOrder',
            'Site' => $Shop,
            'GUID' => '1234-5678-910',
            'BillingAddress' => '',
            'ShippingAddress' => '',
            'Customer' => $Customer,
            'Comment' => 'TestComment',
            'CustomerComment' => 'TestCustomerComment',
            'User' => undef,
            'LineItemContainer' => $LineItemContainer,
            'CreationDate' => $self->_createDateTime(11, 45),
            'DispatchedOn' => $self->_createDateTime(12, 44),
            'ArchivedOn' => $self->_createDateTime(13, 43),
            'CancelledOn' => $self->_createDateTime(14, 42),
            'ClosedOn' => $self->_createDateTime(15, 41),
            'PaidOn' => $self->_createDateTime(16, 40),
            'ReturnedOn' => $self->_createDateTime(17, 39),
            'ViewedOn' => $self->_createDateTime(18, 38),
            'PendingOn' => $self->_createDateTime(19, 37),
            'ShippedOn' => $self->_createDateTime(20, 36),
            'InvoicedOn' => $self->_createDateTime(21, 35),
        });
        return $MockifyEpagesObject->getMockObject();
    });
    $Mockery->register('Order2', ['Shop', 'Customer', 'LineItemContainer'], sub {
        my ($Shop, $Customer, $LineItemContainer) = @_;
        my $MockifyEpagesObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Order::API::Object::CustomerOrder', {
            'Alias' => 'AnotherOrder',
            'Site' => $Shop,
            'GUID' => '6346-233333-3634',
            'BillingAddress' => '',
            'ShippingAddress' => '',
            'Customer' => $Customer,
            'Comment' => 'AComment',
            'CustomerComment' => 'ACustomerComment',
            'User' => undef,
            'LineItemContainer' => $LineItemContainer,
            'CreationDate' => $self->_createDateTime(16, 21),
            'DispatchedOn' => $self->_createDateTime(15, 22),
            'ArchivedOn' => $self->_createDateTime(14, 23),
            'CancelledOn' => $self->_createDateTime(13, 24),
            'ClosedOn' => $self->_createDateTime(12, 25),
            'PaidOn' => $self->_createDateTime(11, 26),
            'ReturnedOn' => $self->_createDateTime(10, 27),
            'ViewedOn' => $self->_createDateTime(9, 28),
            'PendingOn' => $self->_createDateTime(8, 29),
            'ShippedOn' => $self->_createDateTime(7, 30),
            'InvoicedOn' => $self->_createDateTime(6, 31),
        });
        return $MockifyEpagesObject->getMockObject();
    });
    ## use critic (ProhibitMagicNumbers)
    $Mockery->register('Shop', [], sub {
        my $MockifyEpagesObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {
            'Alias' => 'TestShop',
            'LocaleID' => 'de_DE',
            'ID' => '123-456',
        });
        $MockifyEpagesObject->addMockWithReturnValueAndParameterCheck('existsSiteLocale', 1, [{'string' => 'en_GB'}]);
        return $MockifyEpagesObject->getMockObject();
    });
    $Mockery->register('LineItemContainer', [], sub {
        my $MockifyEpagesObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer', {
            'Alias' => 'TestLineItemContainer',
            'CurrencyID' => 'EUR',
            'TaxModel' => 'GROSS',
            'GrandTotal' => '123.45',
            'TotalBeforeTax' => '56',
            'TotalTax' => '23',
        });
        return $MockifyEpagesObject->getMockObject();
    });
    $Mockery->register('Customer', [], sub {
        my $MockifyEpagesObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Customer::API::Object::Customer', {
            'GUID' => '4535-24535443-2343',
        });
        return $MockifyEpagesObject->getMockObject();
    });
    $Mockery->register('OrderService', [], sub {
        my ($OrderService) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Order::API::OrderService', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('getOrderData', $self->_createOrderData(), ['object', 'string']);
        return $Mockify->getMockObject();
    });
    $Mockery->register('Locale', [], sub {
        my $MockifyEpagesObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Locale', {
            'Alias' => 'TestLocale',
            'LanguageID' => 123,
        });
        return $MockifyEpagesObject->getMockObject();
    });
    $Mockery->register('Class', [], sub {
        my $MockifyEpagesObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Class', {
        });
        return $MockifyEpagesObject->getMockObject();
    });
    $Mockery->register('PagerSearch', ['Order', 'Order2'], sub {
        my ($Order, $Order2) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Core::API::Object::BaseObject', []);
        my $PagerSearch = $Mockify->getMockObject();
        $PagerSearch->mock('items', sub {
            return [$Order, $Order2];
        });
        $PagerSearch->{'ItemCount'} = 2;
        return $PagerSearch;
    });
    $Mockery->register('LineItemContainerData', ['Order'], sub {
        my ($Order) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Order::API::LineItemContainerData', [$Order]);
        $Mockify->addMockWithReturnValueAndParameterCheck('getLineItemContainerData', 'lineItemContainerData', ['string', 'string']);
        $Mockify->addMockWithReturnValueAndParameterCheck('getShippingData', 'shippingData', ['string', 'string']);
        $Mockify->addMockWithReturnValueAndParameterCheck('getPaymentData', 'paymentData', ['string', 'string']);
        return $Mockify->getMockObject();
    });

    return $Mockery->assemble();
}



sub _createDateTime {
    my $self = shift;
    my ($Hour, $Minute) = @_;
    my $DateTime = DateTime->new(
        year       => 2015,
        month      => 4,
        day        => 21,
        hour       => $Hour,
        minute     => $Minute,
        second     => 3,
        nanosecond => 0,
        time_zone  => 'Europe/Berlin',
    );
    return $DateTime;
}



sub _createOrderData {
    return {
        'page' => 1,
        'items' => [
            {
                'paidOn' => '2015-04-21T14:40:03Z',
                'paymentData' => 'paymentData',
                'currencyId' => 'EUR',
                'orderId' => '1234-5678-910',
                'taxModel' => 'NET',
                'orderNumber' => 'TestOrder',
                'customerComment' => 'TestCustomerComment',
                'internalNote' => 'TestComment',
                'cancelledOn' => '2015-04-21T12:42:03Z',
                'shippingData' => 'shippingData',
                'viewedOn' => '2015-04-21T16:38:03Z',
                'shippingAddress' => undef,
                'invoicedOn' => '2015-04-21T19:35:03Z',
                'dispatchedOn' => '2015-04-21T10:44:03Z',
                'returnedOn' => '2015-04-21T15:39:03Z',
                'closedOn' => '2015-04-21T13:41:03Z',
                'archivedOn' => '2015-04-21T11:43:03Z',
                'grandTotal' => '123.45',
                'totalBeforeTax' => '56',
                'customerId' => '4535-24535443-2343',
                'totalTax' => '23',
                'creationDate' => '2015-04-21T09:45:03Z',
                'pendingOn' => '2015-04-21T17:37:03Z',
                'billingAddress' => undef,
                'shippedOn' => '2015-04-21T18:36:03Z'
            },
            {
                'paidOn' => '2015-04-21T09:26:03Z',
                'paymentData' => 'paymentData',
                'currencyId' => 'EUR',
                'orderId' => '6346-233333-3634',
                'taxModel' => 'NET',
                'orderNumber' => 'AnotherOrder',
                'customerComment' => 'ACustomerComment',
                'internalNote' => 'AComment',
                'cancelledOn' => '2015-04-21T11:24:03Z',
                'shippingData' => 'shippingData',
                'viewedOn' => '2015-04-21T07:28:03Z',
                'shippingAddress' => undef,
                'invoicedOn' => '2015-04-21T04:31:03Z',
                'dispatchedOn' => '2015-04-21T13:22:03Z',
                'returnedOn' => '2015-04-21T08:27:03Z',
                'closedOn' => '2015-04-21T10:25:03Z',
                'archivedOn' => '2015-04-21T12:23:03Z',
                'grandTotal' => '123.45',
                'totalBeforeTax' => '56',
                'customerId' => '4535-24535443-2343',
                'totalTax' => '23',
                'creationDate' => '2015-04-21T14:21:03Z',
                'pendingOn' => '2015-04-21T06:29:03Z',
                'billingAddress' => undef,
                'shippedOn' => '2015-04-21T05:30:03Z'
            }
        ],
        'resultsPerPage' => 10,
        'results' => 2
    };
}

__PACKAGE__->runScript();