#========================================================================================
# package      DE_EPAGES::PayPalPlus::t::unit::CreatePayment_RepositoryLineItem
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit tests for CreatePayment::LineItemRequestDAO
#========================================================================================
package DE_EPAGES::PayPalPlus::t::unit::CreatePayment_RepositoryLineItem;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::DTO::LineItem;
use DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::LineItem;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::Mockery;

use Test::More;
use strict;

sub test {
    my $self = shift;

    $self->test_Successful_CollectData();
    $self->test_Successful_CollectData_With_LineItemShippingOption();

    done_testing();
}

sub test_Successful_CollectData {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $LineItemProduct = $Mockery->get('LineItemProduct');

    my $LineItemRequestDAO = DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::LineItem->new();
    ok($LineItemRequestDAO->collectData($LineItemProduct), "$SubTestName - test if the collect is successful");

    my $LineItemDTO = $LineItemRequestDAO->getLineItemDTO();
    is($LineItemDTO->name() , '3,01 ml MyProductName', "$SubTestName - test if LineItem name was set correctly");
    is($LineItemDTO->price() , '242.54', "$SubTestName - test if LineItem price was set correctly");
    is($LineItemDTO->quantity() , '1', "$SubTestName - test if LineItem quantity was set correctly");
    is($LineItemDTO->currencyID() , undef, "$SubTestName - test if LineItem currencyID was not set");

    my $hExpectedLineItem = {
          'currencyID' => undef,
          'quantity' => 1,
          'name' => '3,01 ml MyProductName',
          'price' => '242.54',
    };
    is_deeply($LineItemDTO->toHash() , $hExpectedLineItem, "$SubTestName - test if LineItem hash was set correctly");
}

sub test_Successful_CollectData_With_LineItemShippingOption {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $LineItemShippingOption = $Mockery->get('LineItemShippingOption');

    my $LineItemRequestDAO = DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::LineItem->new();
    ok($LineItemRequestDAO->collectData($LineItemShippingOption), "$SubTestName - test if the collect is successful");

    my $LineItemDTO = $LineItemRequestDAO->getLineItemDTO();
    is($LineItemDTO->name() , 'MyShippingOptionName', "$SubTestName - test if LineItem name was set correctly");
    is($LineItemDTO->price() , '2.99', "$SubTestName - test if LineItem price was set correctly");
    is($LineItemDTO->quantity() , '1', "$SubTestName - test if LineItem quantity was set correctly");
    is($LineItemDTO->currencyID() , undef, "$SubTestName - test if LineItem currencyID was not set");

    my $hExpectedLineItem = {
          'currencyID' => undef,
          'quantity' => 1,
          'name' => 'MyShippingOptionName',
          'price' => '2.99',
    };
    is_deeply($LineItemDTO->toHash() , $hExpectedLineItem, "$SubTestName - test if LineItem hash was set correctly");
}

#---------------------------------------------------------------------------
#  Helpers
#---------------------------------------------------------------------------
sub _createMocks {
my $self = shift;
    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();
    $Mockery->register('SiteLocale', [], sub {
        my $hAttributes = {};
        my $MockifyEpagesObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::Object::API::Object::SiteLocale', $hAttributes);
        $MockifyEpagesObject->addMockWithReturnValueAndParameterCheck('formatFloat', '3,01', [{'string'=>'3.01'}]);
        return $MockifyEpagesObject->getMockObject();
    });
    $Mockery->register('SiteLocaleShippingOption', [], sub {
        my $hAttributes = {};
        my $MockifyEpagesObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::Object::API::Object::SiteLocale', $hAttributes);
        $MockifyEpagesObject->addMockWithReturnValueAndParameterCheck('formatFloat', '1', [{'string'=>'1'}]);
        return $MockifyEpagesObject->getMockObject();
    });
    $Mockery->register('Site', ['SiteLocale'], sub {
        my ($SiteLocale) = @_;
        my $hAttributes = {};
        my $MockifyEpagesObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
        $MockifyEpagesObject->addMockWithReturnValueAndParameterCheck('siteLocale', $SiteLocale, [{'string'=>'de_DE'}]);
        return $MockifyEpagesObject->getMockObject();
    });
    $Mockery->register('SiteShippingOption', ['SiteLocaleShippingOption'], sub {
        my ($SiteLocale) = @_;
        my $hAttributes = {};
        my $MockifyEpagesObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
        $MockifyEpagesObject->addMockWithReturnValueAndParameterCheck('siteLocale', $SiteLocale, [{'string'=>'de_DE'}]);
        return $MockifyEpagesObject->getMockObject();
    });
    $Mockery->register('OrderUnit', [], sub {
        my $hAttributes = {
            'Alias' => 'millilitre',
        };
        my $DefaultLanguageID = 0;
        my $MockifyEpagesObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::Object::API::Object::UnitOfMeasurement', $hAttributes , $DefaultLanguageID);
        $MockifyEpagesObject->addLanguageDependentAttribute('NameOrAlias', {
            '2' => 'ml'
        });
        return $MockifyEpagesObject->getMockObject();
    });
    $Mockery->register('LineItemContainer', [], sub {
        my $hAttributes = {
            'Alias' => 'MyContainer',
            'CurrencyID' => 'EUR',
            'TaxModel' => 1,
            'LanguageID' => 2,
            'LocaleID' => 'de_DE',
        };
        my $LineItemContainer = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes );
        return $LineItemContainer;
    });
    $Mockery->register('Product', [], sub {
        my $hAttributes = {
            'Alias' => 'MyProductAlias',
            'Name' => 'MyProductName',
        };
        my $Product = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Product::API::Object::Product', $hAttributes );
        return $Product;
    });
    $Mockery->register('ShippingOption', [], sub {
        my $hAttributes = {
            'Alias' => 'MyShippingOptionAlias',
            'Name' => 'MyShippingOptionName',
        };
        my $ShippingOption = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::ShippingOption::API::Object::ShippingOption', $hAttributes );
        return $ShippingOption;
    });
    $Mockery->register('LineItemProduct', ['Product', 'LineItemContainer', 'OrderUnit', 'Site'], sub {
        my ($Product, $LineItemContainer, $OrderUnit, $Site) = @_;
        my $hAttributes = {
            'Alias' => 'MyLineItemProduct',
            'LineItemPrice' => 242.545,
            'Quantity' => 3.01,
            'Product' => $Product,
            'LineItemContainer' => $LineItemContainer,
            'OrderUnit' => $OrderUnit,
        };
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::Product::API::Object::LineItemProduct', $hAttributes );
        $Mockify->addMockWithReturnValue('getSite', $Site);
        return $Mockify->getMockObject();
    });
    $Mockery->register('LineItemShippingOption', ['ShippingOption', 'LineItemContainer', 'SiteShippingOption'], sub {
        my ($ShippingOption, $LineItemContainer, $Site) = @_;
        my $hAttributes = {
            'Alias' => 'MyLineItemShippingOption',
            'LineItemPrice' => 2.99,
            'Quantity' => 1,
            'ShippingOption' => $ShippingOption,
            'LineItemContainer' => $LineItemContainer,
        };
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::ShippingOption::API::Object::LineItemShippingOption', $hAttributes );
        $Mockify->addMockWithReturnValue('getSite', $Site);
        return $Mockify->getMockObject();
    });

    return $Mockery->assemble();
}

__PACKAGE__->runScript();