#========================================================================================
# package      DE_EPAGES::PayPalPlus::t::unit::CreatePayment_RepositoryRequest
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit tests for CreatePayment::RequestDAO
#========================================================================================
package DE_EPAGES::PayPalPlus::t::unit::CreatePayment_RepositoryRequest;
use base DE_EPAGES::Core::API::Test;


use DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::DTO::LineItem;
use DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::DTO::Request;
use DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::LineItem;
use DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::Request;

use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify;

use Test::More;
use strict;

sub test {
    my $self = shift;

    $self->test_Successful_CollectData();
    $self->test_Successful_CollectData_NoShippingMethod();
    $self->test_Successful_CollectData_NetShop();

    done_testing();
}

## no critic (ProhibitLongLines)
sub test_Successful_CollectData {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_buildBasket_WithShippingMethod();

    my $CreatePayment_RepositoryRequest = $self->_createRequestDAO();
    ok($CreatePayment_RepositoryRequest->collectData($Basket), "$SubTestName - test if the collect is successful");
    my $CreatePaymentRequestDTO = $CreatePayment_RepositoryRequest->getRequestDTO();

    is_deeply($CreatePaymentRequestDTO->toHash() , $self->_expectedHash(), "$SubTestName - test if the toHash was build correctly");

    my $LineItems = $CreatePaymentRequestDTO->lineItems();
    my $LineItem1 = @{$LineItems}[0];
    is($LineItem1->{'currencyID'} , 'EUR', "$SubTestName - test if the currencyID was setted correctly at the first LineItemDTO");
    my $LineItem2 = @{$LineItems}[1];
    is($LineItem2->{'currencyID'} , 'EUR', "$SubTestName - test if the currencyID was setted correctly at the second LineItemDTO");
}

#----------------------------------------------------------------------------------------
sub test_Successful_CollectData_NoShippingMethod {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_buildBasket_WithoutShippingMethod();

    my $CreatePayment_RepositoryRequest = $self->_createRequestDAO();
    ok($CreatePayment_RepositoryRequest->collectData($Basket), "$SubTestName - test if the collect is successful");
    my $CreatePaymentRequestDTO = $CreatePayment_RepositoryRequest->getRequestDTO();

    is($CreatePaymentRequestDTO->shippingPrice() , '0', "$SubTestName - test if the shipping price was setted to 0 if there is no shippingMethod set.");
;
}
#----------------------------------------------------------------------------------------
sub test_Successful_CollectData_NetShop {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_buildBasket_NetShop();

    my $CreatePayment_RepositoryRequest = $self->_createRequestDAO();
    ok($CreatePayment_RepositoryRequest->collectData($Basket), "$SubTestName - test if the collect is successful");
    my $CreatePaymentRequestDTO = $CreatePayment_RepositoryRequest->getRequestDTO();

    is($CreatePaymentRequestDTO->taxPrice() , 9.57, "$SubTestName - test if the tax price was send in case of netShop net shop");
}
#---------------------------------------------------------------------------
#  Mockify helpers
#---------------------------------------------------------------------------
sub _createRequestDAO {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::Request', []);
    $Mockify->addMockWithReturnValue('_createLineItemRepository', $self->_createLineItemRepository());
    $Mockify->addMockWithReturnValue('_createDiscount', $self->_createDiscount());

    return $Mockify->getMockObject();
}
#---------------------------------------------------------------------------
sub _createDiscount {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::Discount', []);
    $Mockify->addMockWithReturnValueAndParameterCheck('getDiscountValue', -10, [{ 'object' => 'DE_EPAGES::Order::API::Object::LineItemContainer' }]);
    $Mockify->addMockWithReturnValueAndParameterCheck('getFeeValue', 5, [{ 'object' => 'DE_EPAGES::Order::API::Object::LineItemContainer' }]);
    return $Mockify->getMockObject();
}
#---------------------------------------------------------------------------
sub _createLineItemRepository {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::LineItem', []);
    $Mockify->addMockWithReturnValueAndParameterCheck('collectData', undef, [{'object' => 'DE_EPAGES::Product::API::Object::LineItemProduct'}]);

    my $LineItemDTO  = DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::DTO::LineItem->new();
    $LineItemDTO->price('9.15');
    $LineItemDTO->quantity('3');
    $Mockify->addMockWithReturnValue('getLineItemDTO', $LineItemDTO);

    return $Mockify->getMockObject();
}
#---------------------------------------------------------------------------
#  Helpers
#---------------------------------------------------------------------------
sub _buildBasket_WithShippingMethod {
    my $self = shift;
    my $IsGrossShop = 1;
    my $hAttributes = {
        'LineItemContainer' => $self->_createLineItemContainer($IsGrossShop),
    };
    my $Basket = $self->_createBasket($hAttributes);
    return $Basket;
}
#---------------------------------------------------------------------------
sub _buildBasket_WithoutShippingMethod {
    my $self = shift;
    my $IsGrossShop = 1;
    my $hAttributes = {
        'LineItemContainer' => $self->_createLineItemContainer_withoutShippingMethod($IsGrossShop),
    };
    my $Basket = $self->_createBasket($hAttributes);
    return $Basket;
}
sub _buildBasket_NetShop {
    my $self = shift;
    my $IsGrossShop = 0;
    my $hAttributes = {
        'LineItemContainer' => $self->_createLineItemContainer($IsGrossShop),
    };
    my $Basket = $self->_createBasket($hAttributes);
    return $Basket;
}
#---------------------------------------------------------------------------
sub _createBasket {
    my $self = shift;
    my ($hAdditionalAttributes) = @_;
    my $hAttributes = {
        'Alias' => 'Basket',
        %{$hAdditionalAttributes},
    };
    my $Basket = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::Basket', $hAttributes );
    return $Basket;
}
#---------------------------------------------------------------------------
sub _createLineItemContainer {
    my $self = shift;
    my ($IsGrossShop) = @_;
    my $hAttributes = {
        'Alias' => 'MyContainer',
        'GrandTotal' => 75.57,
        'TotalTax' => 9.57,
        'CurrencyID' => 'EUR',
        'LineItems' => $self->_createLineItems(),
        'Payment' => $self->_createLineItemPaymentPayPalPlus(),
        'Shipping' => $self->_createLineItemShipping(),
        'TaxModel' => $IsGrossShop,
    };
    my $LineItemContainer = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes );
    return $LineItemContainer;
}
#---------------------------------------------------------------------------
sub _createLineItemContainer_withoutShippingMethod {
    my $self = shift;
    my ($IsGrossShop) = @_;
    my $hAttributes = {
        'Alias' => 'MyContainer',
        'GrandTotal' => 80.57,
        'CurrencyID' => 'EUR',
        'LineItems' => $self->_createLineItems(),
        'Payment' => $self->_createLineItemPaymentPayPalPlus(),
        'Shipping' => undef,
        'TaxModel' => $IsGrossShop,
    };
    my $LineItemContainer = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes );
    return $LineItemContainer;
}
#---------------------------------------------------------------------------
sub _createLineItems {
    my $self = shift;
    my $hAttributes1 = {
        'Alias' => 'MyProduct1',
    };
    my $LineItem1 = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Product::API::Object::LineItemProduct', $hAttributes1 );
    my $hAttributes2 = {
        'Alias' => 'MyProduct2',
    };
    my $LineItem2 = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Product::API::Object::LineItemProduct', $hAttributes2 );
    return [$LineItem1, $LineItem2];
}
#---------------------------------------------------------------------------
sub _createLineItemPaymentPayPalPlus {
    my $self = shift;
    my $hAttributes = {
        'Alias' => 'LineItemPaymentPayPalPlus',
        'GUID' => 'myGUID',
        'PaymentMethod' => $self->_createPaymentMethodPayPalPlus(),
    };
    my $LineItemPaymentPayPalPlus = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::PayPalPlus::API::Object::LineItemPaymentPayPalPlus', $hAttributes );
    return $LineItemPaymentPayPalPlus;
}
#---------------------------------------------------------------------------
sub _createLineItemShipping {
    my $self = shift;
    my $hAttributes = {
        'Alias' => 'LineItemShipping',
        'LineItemPrice' => 25.67,
    };
    my $LineItemShipping = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::LineItemShipping', $hAttributes );
    return $LineItemShipping;
}
#---------------------------------------------------------------------------
sub _createPaymentMethodPayPalPlus {
    my $self = shift;
    my $hAttributes = {
        'Alias' => 'PaymentMethodPayPalPlus',
        'ExperienceProfileId' => 'myProfileID',
        'IsInTestMode' => 1,
        'MerchantClientId' => 'MyDummyMerchantClientId',
        'PaymentType' => $self->_createPaymentType(),
    };
    my $PaymentMethodPayPalPlus = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus', $hAttributes );
    return $PaymentMethodPayPalPlus;
}
#---------------------------------------------------------------------------
sub _createPaymentType {
    my $self = shift;

    my $hAttributes = {
        'ProviderClientId' => 'MyDummyProviderClientId',
        'ProviderClientSecret' => 'MyDummyProviderClientSecret',
        'PayPalPartnerAttributionId' => 'MyAttributionId',
    };
    my $PaymentType = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Payment::API::Object::PaymentType', $hAttributes );

    return $PaymentType;
}
#---------------------------------------------------------------------------
#  Expected results
#---------------------------------------------------------------------------
sub _expectedHash {
    my $self = shift;
    my $hRequest = {
          'basketCurrency' => 'EUR',
          'returnUrl' => undef,
          'basketPrice' => '75.57',
          'subTotalPrice' => '54.9',
          'taxPrice' => undef,
          'shippingPrice' => '25.67',
          'discountPrice' => '-10',
          'handlingFeePrice' => '5',
          'experienceProfileId' => 'myProfileID',
          'lineItems' => [
                           {
                             'currencyID' => 'EUR',
                             'quantity' => '3',
                             'name' => undef,
                             'price' => '9.15',
                           },
                           {
                             'currencyID' => 'EUR',
                             'quantity' => '3',
                             'name' => undef,
                             'price' => '9.15',
                           }
                         ],
          'lineItemPaymentGUID' => 'myGUID',
          'cancelUrl' => undef
    };
    return $hRequest;
}


## use critic (ProhibitLongLines)
__PACKAGE__->runScript();