#========================================================================================
# package      DE_EPAGES::PayPalPlus::t::unit::PaymentMethodPayPalPlus
# base         base DE_EPAGES::Core::API::Test
#----------------------------------------------------------------------------------------
# description  Unit test for PaymentMethodPayPalPlus
#========================================================================================
package DE_EPAGES::PayPalPlus::t::unit::PaymentMethodPayPalPlus;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::More;
use strict;

sub test {
    my $self = shift;

    $self->test_setDefaultNameForAllLanguages_ValidDefaultName();
    $self->test_setDefaultNameForAllLanguages_InvalidDefaultName();

    done_testing();
}

sub test_setDefaultNameForAllLanguages_ValidDefaultName {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PaymentType = $self->_createPaymentType();
    my $SUT = $self->_buildSUT($PaymentType);

    is($SUT->get('Name', 1), 'InitialName', "$SubTestName - Test if name DE is initialized properly.");
    is($SUT->get('Name', 2), 'InitialName', "$SubTestName - Test if name EN is initialized properly.");

    $SUT->_setDefaultNameForAllLanguages(); ## no critic (ProhibitPrivateSubroutineUse)

    is($SUT->get('Name', 1), 'DefaultNameDE', "$SubTestName - Test if name DE was set to default name when present.");
    is($SUT->get('Name', 2), 'DefaultNameEN', "$SubTestName - Test if name EN was set to default name when present.");
}

sub test_setDefaultNameForAllLanguages_InvalidDefaultName {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PaymentType = $self->_createPaymentTypeMissingLanguageAttribute();
    my $SUT = $self->_buildSUT($PaymentType);

    is($SUT->get('Name', 1), 'InitialName', "$SubTestName - Test if name DE is initialized properly.");
    is($SUT->get('Name', 2), 'InitialName', "$SubTestName - Test if name EN is initialized properly.");

    $SUT->_setDefaultNameForAllLanguages(); ## no critic (ProhibitPrivateSubroutineUse)

    is($SUT->get('Name', 1), 'DefaultNameDE', "$SubTestName - Test if name DE was set to default name when present.");
    is($SUT->get('Name', 2), 'InitialName', "$SubTestName - Test if name EN was not set to default name when not present.");
}

sub _createPaymentType {
    my $self = shift;
    my $hAttributes = {
        1 => 'DefaultNameDE',
        2 => 'DefaultNameEN',
    };

    return $self->_buildPaymentType($hAttributes);
}

sub _createPaymentTypeMissingLanguageAttribute {
    my $self = shift;
    my $hAttributes = {
        1 => 'DefaultNameDE',
        2 => undef,
    };

    return $self->_buildPaymentType($hAttributes);
}

sub _buildPaymentType {
    my $self = shift;
    my ($hAttributes) = @_;

    my $PackageName = 'DE_EPAGES::Payment::API::Object::PaymentType';
    my $DefaultLanguageId = 1;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new($PackageName, {}, $DefaultLanguageId);
    $Mockify->addLanguageDependentAttribute('PaymentMethodDefaultName', $hAttributes);

    return $Mockify->getMockObject();
}

sub _createShop {
    my $self = shift;

    my $PackageName = 'DE_EPAGES::Payment::API::Object::PaymentType';
    my $hAttributes = {
        'Languages' => [{'LanguageID' => 1}, {'LanguageID' => 2}],
    };
    my $DefaultLanguageId = 1;

    return DE_EPAGES::Test::Mock::MockifyEpagesObject->new($PackageName, $hAttributes, $DefaultLanguageId)->getMockObject();
}

sub _buildSUT {
    my $self = shift;
    my ($PaymentType) = @_;

    my $PackageName = 'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus';
    my $hAttributes = {
        'Shop' => $self->_createShop(),
        'PaymentType' => $PaymentType,
    };
    my $DefaultLanguageId = 1;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new($PackageName, $hAttributes, $DefaultLanguageId);
    $Mockify->addLanguageDependentAttribute('Name', {
        1 => 'InitialName',
        2 => 'InitialName',
    });

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();