#========================================================================================
# package      DE_EPAGES::PayPalPlus::t::unit::Webservice_Base_Repository
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for Webservice::Base::Repository
#========================================================================================
package DE_EPAGES::PayPalPlus::t::unit::Webservice_Base_Repository;
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::PayPalPlus::API::Webservice::Base::Repository;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::More;
use Test::Exception;
use strict;

sub test {
    my $self = shift;

    $self->test_BaseWebservice_InTestMode();
    $self->test_BaseWebservice_InLiveMode();

    done_testing();
}

sub test_BaseWebservice_InTestMode {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsInTestMode = 1;
    my $Repository = DE_EPAGES::PayPalPlus::API::Webservice::Base::Repository->new();
    my $PaymentMethod = $self->_createPaymentMethod($IsInTestMode);
    $Repository->buildCredentialDTO($PaymentMethod);
    my $CredentialDTO = $Repository->getCredentialDTO();

    is($CredentialDTO->isInTestMode() ,'true',"$SubTestName - check if in test mode");
    is($CredentialDTO->merchantClientId() , 'Dummy_MerchantClientId',"$SubTestName - check if merchant client id");
    is($CredentialDTO->providerClientSecret() ,'Dummy_ProviderClientSecret',"$SubTestName - check for ProviderClientSecret");
    is($CredentialDTO->providerClientId() ,'Dummy_ProviderClientSecret',"$SubTestName - check for ProviderClientSecret");
    is($CredentialDTO->payPalPartnerAttributionId() ,'MyAttributionId',"$SubTestName - check for PayPalPartnerAttributionId");
}

sub test_BaseWebservice_InLiveMode {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsInTestMode = 0;
    my $Repository = DE_EPAGES::PayPalPlus::API::Webservice::Base::Repository->new();
    my $PaymentMethod = $self->_createPaymentMethod($IsInTestMode);
    $Repository->buildCredentialDTO($PaymentMethod);
    my $CredentialDTO = $Repository->getCredentialDTO();

    is($CredentialDTO->isInTestMode() ,'false',"$SubTestName - check if in live mode");
    is($CredentialDTO->merchantClientId() , 'Dummy_MerchantClientId',"$SubTestName - check if merchant client id");
    is($CredentialDTO->providerClientSecret() ,'Dummy_ProviderClientSecret',"$SubTestName - check for ProviderClientSecret");
    is($CredentialDTO->providerClientId() ,'Dummy_ProviderClientSecret',"$SubTestName - check for ProviderClientSecret");
    is($CredentialDTO->payPalPartnerAttributionId() ,'MyAttributionId',"$SubTestName - check for PayPalPartnerAttributionId");
}
#----------------------------------------------------------------------------------------
#  helpers
#----------------------------------------------------------------------------------------
sub _createPaymentMethod {
    my $self = shift;
    my ($IsInTestMode) = @_;

    my $hAttributes = {
        'IsInTestMode' => $IsInTestMode,
        'MerchantClientId' => 'Dummy_MerchantClientId',
        'PaymentType' => $self->_createPaymentType(),
    };
    my $PaymentMethod = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus', $hAttributes );

    return $PaymentMethod;
}
#----------------------------------------------------------------------------------------
sub _createPaymentType {
    my $self = shift;

    my $hAttributes = {
        'ProviderClientSecret' => 'Dummy_ProviderClientSecret',
        'ProviderClientId' => 'Dummy_ProviderClientSecret',
        'PayPalPartnerAttributionId' => 'MyAttributionId'
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Payment::API::Object::PaymentType', $hAttributes );
}
__PACKAGE__->runScript();